/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share;

import java.io.File;

import org.alfresco.bm.event.AbstractEventProcessor;
import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.session.SessionService;
import org.alfresco.bm.user.UserData;
import org.alfresco.bm.user.UserDataService;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openqa.selenium.OutputType;
import org.openqa.selenium.TakesScreenshot;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.remote.Augmenter;

/**
 * Abstract for all event process that use share
 * with {@link WebDriver}. 
 * 
 * @author Michael Suzuki
 * @since 1.0
 * 
 */
public abstract class AbstractShareEventProcessor extends AbstractEventProcessor
{
    protected final Log logger;
    protected UserDataService userDataService;
    protected SessionService sessionService;

    /**
     * The name of the next event to follow upon completing the current event.
     */
    protected String eventNameActionComplete;

    /**
     * Constructor.
     * @param userDataService user data service
     * @param eventNameActionComplete String next event name
     */
    public AbstractShareEventProcessor(UserDataService userDataService, String eventNameActionComplete)
    {
        this.userDataService = userDataService;
        this.eventNameActionComplete = eventNameActionComplete;
        this.logger = LogFactory.getLog(getClass());
    }

    /**
     * Set the service required to end browser sessions
     */
    public void setSessionService(SessionService sessionService)
    {
        this.sessionService = sessionService;
    }

    /**
     * Set the name of the event to follow.
     * @param eventNameActionComplete
     */
    public void setEventNameActionComplete(String eventNameActionComplete)
    {
        this.eventNameActionComplete = eventNameActionComplete;
    }
    
    /**
     * Process a share based event.
     * @param event                 the origininating event (not normally required)
     * @param eventData             the share event data entity 
     * @return {@link EventResult}  the result of the process (includes errors and next events)
     */
    protected abstract EventResult processEvent(Event event, ShareEventData eventData) throws Exception;
    
    @Override
    public EventResult processEvent(final Event event) throws Exception
    {
        EventResult result = null;
        ShareEventData data = null;
        String sessionId = event.getSessionId();
        try
        {
            data = (ShareEventData) event.getData();
            if (data == null)
            {
                throw new UnsupportedOperationException("Unable to process event, share data is required: " + event);
            }
            result = processEvent(event, data);
        }
        catch (Exception t)
        {
            WebDriver driver = null == data ? null : data.getWebDriver();
            // Take a screenshot, if possible
            File screenshot = null;
            if (driver != null)
            {
                try
                {
                    if(driver instanceof TakesScreenshot) 
                    {
                         screenshot = ((TakesScreenshot) driver).getScreenshotAs(OutputType.FILE);
                    }
                    WebDriver augmentedDriver = new Augmenter().augment(driver);
                    screenshot = ((TakesScreenshot) augmentedDriver).getScreenshotAs(OutputType.FILE);
                    screenshot.deleteOnExit();
                }
                catch (UnsupportedOperationException e)
                {
                    // This is OK.  No need to log it
                }
                catch (Exception e)
                {
                    // Could not take it but don't lose the actual problem exception
                    logger.error(
                            "Unable to take screenshot of failure: \n" +
                            "   " + event);
                }
            }
            String webDriverStr = "WebDriver.toString() failed";
            try
            {
                webDriverStr = (data == null) ? null : "" + data.getWebDriver();
            }
            catch (UnsupportedOperationException e)
            {
                // Again.  This happens and should not mask our actual problem
            }
            throw new ShareEventProcessException(
                            "Failed to process share event: \n" +
                            "   Event:      " + event + "\n" +
                            "   Data:       " + data + "\n" +
                            "   WebDriver:   " + webDriverStr + "\n" +
                            "   Screenshot: " + (screenshot == null ? "<none>" : screenshot.getPath()) + "\n" +
                            "   Error:      " + t.getMessage(),
                            t);
        }
        finally
        {
            if (result == null || !result.isSuccess())
            {
                WebDriver driver = (null == data) ? null : data.getWebDriver();
                closeWebDriver(driver, sessionId);
            }

        }
        return result;
    }
    
    /**
     * Shut {@link WebDriver} off.
     * @param driver             {@link WebDriver} the unmanned browser
     * @param sessionId         the session to close
     */
    protected void closeWebDriver(WebDriver driver, String sessionId)
    {
        if (driver != null)
        {
            try
            {
                driver.quit();
            }
            catch (Exception e)
            {
                // We have to accept this and rely on the grid to clean things up
                logger.error("Failed to close browser connection: " + e.getMessage());
            }
        }
        if (sessionId != null)
        {
            sessionService.endSession(sessionId);
        }
    }
    
    /**
     * Extracts user entity from data service.
     * @param username String user identifier
     * @return {@link UserData} user entity
     * @throws UserNotFoudException if user name did not match a user
     */
    public UserData getUserData(final String username) throws UserNotFoudException
    {
        UserData user = userDataService.findUserByUsername(username);
        if (user == null) throw new UserNotFoudException();
        return user;
    }
}
