/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share;

import java.util.Collections;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.process.share.entity.ShareStartEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.FactorySharePage;
import org.alfresco.po.share.SharePage;
import org.openqa.selenium.WebDriver;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * Process which makes the {@link WebDriver} go to the given Alfresco share URL.
 * 
 * <h1>Input</h1> 
 * 
 * Required a {@link ShareStartEventData} instance that includes the
 * {@link WebDriver} and URL to navigate to.
 * 
 * <h1>Actions</h1>
 *  
 * Uses the Web Drone to navigate to an Alfresco Url. 
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_NAVIGATE_SHARE_COMPLETE}: A {@link ShareEventData} with an initialized
 * {@link ShareEventData#getSharePage() Share page}.
 * 
 * @author Michael Suzuki
 * @since 1.0
 */
public class NavigateToShareEventProcess extends AbstractShareEventProcessor
{
    private static final String ERROR_URL_IS_REQUIRED = "Accessing share event process failed, URL is required";
    private static final String ERROR_WEBDRIVER_IS_REQUIRED = "Accessing share event process failed, WebDriver is required";
    private static final String ERROR_WEBDRIVER_EVENT_DATA_IS_REQUIRED = "Accessing share event process failed, WebDriverEventData is required";
    private static final String EVENT_NAME_NAVIGATE_SHARE_COMPLETE = "share.navigateToShare.complete";
    @Autowired FactorySharePage factorySharePage;
    public NavigateToShareEventProcess(UserDataService userDataService)
    {
        super(userDataService, EVENT_NAME_NAVIGATE_SHARE_COMPLETE);
    }

    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        if (eventData == null || !(eventData instanceof ShareStartEventData)) 
        { 
            return new EventResult(ERROR_WEBDRIVER_EVENT_DATA_IS_REQUIRED, false); 
        }
        ShareStartEventData startEventData = (ShareStartEventData) eventData;

        WebDriver driver = startEventData.getWebDriver();
        if (driver == null) 
        { 
            return new EventResult(ERROR_WEBDRIVER_IS_REQUIRED, false);
        }

        String url = eventData.getUrl();
        if (url == null || url.isEmpty()) 
        {
            return new EventResult(ERROR_URL_IS_REQUIRED, false);
        }

        driver.navigate().to(url);
        SharePage sharePage = (SharePage)factorySharePage.getPage(driver).render(); 
        eventData.setSharePage(sharePage);
        Event nextEvent = new Event(eventNameActionComplete, eventData);
        return new EventResult(
                "Navigated to Share URL: " + url,
                Collections.singletonList(nextEvent),
                true);
    }
}
