/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share;

import java.lang.reflect.Method;
import java.util.Collections;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.Navigation;
import org.alfresco.po.share.SharePage;
import org.alfresco.po.share.site.SiteNavigation;

/**
 * An Alfresco share navigation process that mimics selecting a
 * button on Alfresco's main navigation bar.
 *  
 * <h1>Input</h1> 
 * A {@link SharePage} that is displayed to the user.
 * 
 * <h1>Actions</h1>
 * Uses the {@link Navigation} on the share page to navigate from the current rendered
 * page to the destination page using the main navigation bar.
 * 
 * <h1>Output</h1>
 * {@link #EVENT_NAME_NAVIGATION_COMPLETE}: A Share page after navigation<br/>
 * 
 * @author Michael Suzuki
 * 
 * @since 1.0
 */
public class NavigationEventProcess extends AbstractShareEventProcessor
{
    private final static String EVENT_NAME_NAVIGATION_COMPLETE = "share.navigation.complete";
    private String navigateTo;

    /**
     * Default constructor.
     * 
     * @param navigateTo            The method name to select on the {@link Navigation} for the
     *                              process to eventNavigationActionComplete.
     */
    public NavigationEventProcess(final UserDataService userDataService, final String navigateTo)
    {
        super(userDataService, EVENT_NAME_NAVIGATION_COMPLETE);
        this.navigateTo = navigateTo;
    }
    /**
     * Constructor.
     * @param userDataService {@link UserDataService}
     * @param eventNavigationName the name identifier for the action
     * @param navigateTo the name of method to use on {@link SiteNavigation}
     */
    public NavigationEventProcess(final UserDataService userDataService,final String eventNavigationName, final String navigateTo)
    {
        super(userDataService, eventNavigationName);
        this.navigateTo = navigateTo;
    }
    
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        // Get the button from navigation to click
        if (navigateTo == null || navigateTo.isEmpty())
        { 
            return new EventResult(
                "Navigation eventNavigationActionComplete failed, navigate to parameter not found.",
                Collections.<Event> emptyList(),
                false);
        }
        return navigateTo(eventData);
    }

    /**
     * Site main navigation, the event of selecting on the main navigation.
     * 
     * @param {@link Event} event data
     * @return {@link EventResult} of the eventNavigationActionComplete
     */
    private EventResult navigateTo(final ShareEventData eventData) 
    {
        suspendTimer();
        
        EventResult eventResult = null;
        SharePage page = eventData.getSharePage();
        try
        {
            Navigation nav = page.getNav();
            Method method = nav.getClass().getMethod(navigateTo, new Class[] {});
            SharePage response = (SharePage) method.invoke(nav, new Object[] {});
            
            resumeTimer();
            response = response.render();
            stopTimer();
            
            eventData.setSharePage(response);
            Event nextEvent = new Event(eventNameActionComplete, eventData);
            eventResult = new EventResult(
                    "Share navigation link clicked: " + navigateTo,
                    Collections.singletonList(nextEvent),
                    true);
        }
        catch (NoSuchMethodException e)
        {
            eventResult = new EventResult(
                    "Navigation eventNavigationActionComplete failed, param does not match any existing navigation option",
                    Collections.<Event> emptyList(), false);
        }
   
        catch (Throwable throwable)
        {
            throw new ShareEventProcessException(page.getWebDriver(), throwable);
        }
        return eventResult;
    }
}
