/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share;

import org.alfresco.bm.event.AbstractEventProcessor;
import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.process.share.entity.ShareStartEventData;
import org.alfresco.bm.session.SessionService;
import org.openqa.selenium.WebDriver;
import org.springframework.beans.factory.FactoryBean;

/**
 * <p>Prepare and starts a {@link WebDriver} to be used by a user to navigate share</p>
 * 
 * <h1>Input</h1>
 * 
 * A user name.
 * 
 * <h1>Actions</h1>
 * 
 * WebDriver is created by having an alfresco version and web driver injected, this
 * data is then passed on to the next Event in a {@link Event} wrapper.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_START_WEBDRIVER_COMPLETE}: {@link ShareEventData} instance with active
 * {@link WebDriver} instance.
 *
 * @author Michael Suzuki
 * @since 1.0
 */
public class WebDriverStartEventProcess extends AbstractEventProcessor
{
    private static final String ERROR_NULL_WEBDRIVER = "Factory returned null WebDriver.";
    private static final String EVENT_NAME_START_WEBDRIVER_COMPLETE = "share.startWebDriver.complete";
    
    private final FactoryBean<WebDriver> webDriverFactory;
    private final SessionService sessionService;
    private final String shareUrl;
    private String eventWebDriverStarted;
    
    /**
     * Start {@link WebDriver}, the unmanned web browser.
     * @param webDriverFactory       factory providing new {@link WebDriver} instance
     * @param sessionService         service to record data and times for user sessions
     * @param shareUrl               the Share server URL
     */
    public WebDriverStartEventProcess(
            FactoryBean<WebDriver> webDriverFactory,
            SessionService sessionService,
            String shareUrl)
    {
        this.webDriverFactory = webDriverFactory;
        this.sessionService = sessionService;
        this.shareUrl = shareUrl;
        this.eventWebDriverStarted = EVENT_NAME_START_WEBDRIVER_COMPLETE;
    }

    /**
     * Override the {@link #EVENT_NAME_START_WEBDRIVER_COMPLETE default} event name for completion
     */
    public void setEventWebDriverStarted(String eventWebDriverStarted)
    {
        this.eventWebDriverStarted = eventWebDriverStarted;
    }
    
    @Override
    public EventResult processEvent(Event event) throws Exception
    {
        EventResult result = null;

        String username = (String) event.getData();
        if (username == null)
        {
            return new EventResult("No username supplied.", false);
        }
        
        try
        {
            WebDriver driver = webDriverFactory.getObject();
            if (driver != null)
            {
                // Start a session
                String sessionId = sessionService.startSession(null);
                
                ShareStartEventData data = new ShareStartEventData(username, shareUrl, driver);
                Event eventStartDrone = new Event(eventWebDriverStarted, data);
                eventStartDrone.setSessionId(sessionId);
                result = new EventResult(
                        "Started WebDriver: " + driver.getCurrentUrl(),
                        eventStartDrone);
            }
            else
            {
                return new EventResult(ERROR_NULL_WEBDRIVER, false);
            }
        }
        catch (Throwable e)
        {
            throw new RuntimeException("Unable to start Web Driver using factory: " + webDriverFactory, e);
        }
        return result;
    }

    /**
     * We record and maintain session data for all browsers.
     */
    public void propagateSessionId(Event event, Event nextEvent)
    {
        // Start a recorded session with no initial data
        String sessionId = sessionService.startSession(null);
        nextEvent.setSessionId(sessionId);
    }
}
