/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.search;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.process.share.entity.ShareSearchEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.SharePage;
import org.alfresco.po.share.search.SearchResultsPage;

/**
 * An Alfresco share search event process, mimics the action of entering a term into
 * the search box.
 *  
 * <h1>Input</h1>
 *  
 * {@link ShareSearchEventData} with a {@link SharePage} containing the search box and
 * a {@link ShareSearchEventData#getSearchTerm() search term}. 
 * 
 * <h1>Actions</h1>
 * 
 * Calls the main naviation {@link SearchResultPage search} link.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_SEARCH_COMPLETE}: {@link SearchResultsPage}<br/>
 * 
 * @author Michael Suzuki
 * 
 * @since 1.0
 */
public class SearchEventProcess extends AbstractShareEventProcessor
{
    private static final String ERROR_NO_SEARCH_DATA = "Search action failed; expected ShareSearchEventData.";
    public static final String EVENT_NAME_SEARCH_COMPLETE = "share.search.complete";

    public SearchEventProcess(UserDataService userDataService)
    {
        super(userDataService,EVENT_NAME_SEARCH_COMPLETE);
    }
    
    /**
     * Process which enters the term into search box and
     * performs the search.
     * 
     * @param eventData {@link Event} event information
     * @return {@link EventResult} result of the process
     */
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        if (!(eventData instanceof ShareSearchEventData))
        {
            return new EventResult(ERROR_NO_SEARCH_DATA, false);
        }
        ShareSearchEventData searchEventData = (ShareSearchEventData) eventData;
        String searchTerm = searchEventData.getSearchTerm();
        
        SharePage page = eventData.getSharePage();
        SharePage results = page.getSearch().search(searchTerm).render();
        eventData.setSharePage(results);
        Event nextEvent = new Event(eventNameActionComplete, eventData);
        EventResult result = new EventResult(
                "User '" + eventData.getUsername() + "' searched for: " + searchTerm,
                nextEvent);
        return result;
    }
}
