/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.search;

import java.util.List;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.SharePage;
import org.alfresco.po.share.search.SearchResultPage;
import org.alfresco.po.share.search.SearchResult;
import org.alfresco.po.share.search.SearchResultsPage;
import org.alfresco.po.share.site.document.DocumentDetailsPage;
import org.alfresco.po.share.site.document.DocumentLibraryPage;
import org.alfresco.po.share.site.wiki.WikiPage;

/**
 * Select file event process: mimics the action of selecting an entry
 * on the search results page.
 *  
 * <h1>Input</h1>
 *  
 * {@link ShareEventData} containing {@link SearchResultsPage} and with the
 * {@link ShareEventData#getFilename() filename} populated.
 * 
 * <h1>Actions</h1>
 * 
 * Clicks the file or folder in the search results.  Only after the selection
 * has occured will the type of the selection be known; the output event type
 * will depend on the type of selection made.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_SELECT_ENTRY_COMPLETE}: {@link DocumentDetailsPage} <b>OR</b>
 *      {@link DocumentLibraryPage}in {@link ShareEventData}<br/>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class SelectSearchEntryEventProcess extends AbstractShareEventProcessor
{
    public static final String EVENT_NAME_SELECT_FILE_COMPLETE = "share.search.selectFile.complete";
    public static final String EVENT_NAME_SELECT_FOLDER_COMPLETE = "share.search.selectFolder.complete";
    public static final String EVENT_NAME_SELECT_WIKI_COMPLETE = "share.search.selectWiki.complete";
    public static final String EVENT_NAME_SELECT_UNKNOWN_COMPLETE = "share.search.selectUnknown.complete";

    private String eventNameSelectFileComplete = EVENT_NAME_SELECT_FILE_COMPLETE;
    private String eventNameSelectFolderComplete = EVENT_NAME_SELECT_FOLDER_COMPLETE;
    private String eventNameSelectWikiComplete = EVENT_NAME_SELECT_WIKI_COMPLETE;
    private String eventNameSelectUnknownComplete = EVENT_NAME_SELECT_UNKNOWN_COMPLETE;
    
    public SelectSearchEntryEventProcess(UserDataService userDataService)
    {
        super(userDataService,EVENT_NAME_SELECT_FILE_COMPLETE);
    }

    /**
     * Override the {@link #EVENT_NAME_SELECT_FILE_COMPLETE default} output event name
     * when a file is selected.
     */
    public void setEventNameSelectFileComplete(String eventNameSelectFileComplete)
    {
        this.eventNameSelectFileComplete = eventNameSelectFileComplete;
    }

    /**
     * Override the {@link #EVENT_NAME_SELECT_FOLDER_COMPLETE default} output event name
     * when a folder is selected.
     */
    public void setEventNameSelectFolderComplete(String eventNameSelectFolderComplete)
    {
        this.eventNameSelectFolderComplete = eventNameSelectFolderComplete;
    }

    /**
     * Override the {@link #EVENT_NAME_SELECT_WIKI_COMPLETE default} output event name
     * when a Wiki page is selected.
     */
    public void setEventNameSelectWikiComplete(String eventNameSelectWikiComplete)
    {
        this.eventNameSelectWikiComplete = eventNameSelectWikiComplete;
    }

    /**
     * Override the {@link #EVENT_NAME_SELECT_UNKNOWN_COMPLETE default} output event name
     * when an unrecognised search result is selected.
     */
    public void setEventNameSelectUnknownComplete(String eventNameSelectUnknownComplete)
    {
        this.eventNameSelectUnknownComplete = eventNameSelectUnknownComplete;
    }

    /**
     * Process which selects the file link.
     * 
     * @param eventData {@link Event} event information
     * @return {@link EventResult} result of the process
     */
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        super.suspendTimer();
        SearchResultPage searchResultsPage = (SearchResultPage) eventData.getSharePage();
        List<SearchResult> searchResults = searchResultsPage.getResults();
        int selection = (int)(Math.random() * searchResults.size());

        super.resumeTimer();
        SharePage page = null;
        page = searchResultsPage.selectItem(selection).render();
        stopTimer();
        Event nextEvent = null;
        String msg = null;
        // Now we can find out what was clicked
        eventData.setSharePage(page);
        if (page instanceof DocumentDetailsPage)
        {
            nextEvent = new Event(eventNameSelectFileComplete, eventData);
            msg = "Search result file selected: " + page.getWebDriver().getCurrentUrl();
        }
        else if (page instanceof DocumentLibraryPage)
        {
            nextEvent = new Event(eventNameSelectFolderComplete, eventData);
            msg = "Search result folder selected: " + page.getWebDriver().getCurrentUrl();
        }
        else if (page instanceof WikiPage)
        {
            nextEvent = new Event(eventNameSelectWikiComplete, eventData);
            msg = "Search result Wiki selected: " + page.getWebDriver().getCurrentUrl();
        }
        else
        {
            nextEvent = new Event(eventNameSelectUnknownComplete, eventData);
            msg = "Search result Unknown selected: " + page.getWebDriver().getCurrentUrl();
        }
        
        EventResult result = new EventResult(msg, nextEvent);
        return result;
    }
}
