/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.site;

import java.io.File;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.file.TestFileService;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.ShareEventProcessException;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.process.share.entity.ShareUploadFileEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.site.UploadFilePage;
import org.alfresco.po.share.site.document.DocumentLibraryPage;

/**
 * An Alfresco share file upload process, mimics the action of uploading a file 
 * to the site and completing the file upload form.
 *  
 * <h1>Input</h1> 
 * 
 * {@link UploadFilePage} The page object representing the file upload form to complete.
 * 
 * <h1>Actions</h1>
 * 
 * Populates the file upload form with file name and submits it to upload a file.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_FILE_UPLOAD_COMPLETE}: {@link DocumentLibraryPage} in {@link ShareEventData}<br/>
 * 
 * @author Michael Suzuki
 * 
 * @since 1.0
 */
public class FileUploadEventProcess extends AbstractShareEventProcessor
{
    public static final String ERROR_NO_FILES = "Uploading file action failed.  There are no files available for upload.";
    public static final String EVENT_NAME_FILE_UPLOAD_COMPLETE = "share.doclib.uploadFile.complete";
    
    private final TestFileService testFileService;

    /**
     * @param userDataService           service to provide user data
     * @param testFileService           service to provide test files
     */
    public FileUploadEventProcess(UserDataService userDataService, TestFileService testFileService)
    {
        super(userDataService,EVENT_NAME_FILE_UPLOAD_COMPLETE);
        this.testFileService = testFileService;
    }
    
    /**
     * Process which fills and submits the file upload form.
     * 
     * @param eventData {@link Event} event information
     * @return {@link EventResult} result of the process
     */
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        if (!(eventData instanceof ShareUploadFileEventData))
        {
            throw new ShareEventProcessException("File creation must be done with ShareUploadFileEventData.");
        }
        ShareUploadFileEventData uploadFileEventData = (ShareUploadFileEventData) eventData;
        File file = null;
        String filename = uploadFileEventData.getFilename();
        if (filename == null)
        {
            file = testFileService.getFile();
            if (file == null)
            {
                throw new RuntimeException(ERROR_NO_FILES);
            }
            filename = file.getCanonicalPath();
        }
        else
        {
            file = new File(filename);
        }
        // Make sure the file exists
        if (!file.exists())
        {
            throw new IllegalArgumentException("Upload file does not exist: " + file);
        }
        
        UploadFilePage upLoadPage = (UploadFilePage) eventData.getSharePage();
        DocumentLibraryPage docPage = upLoadPage.uploadFile(filename).render();
        eventData.setSharePage(docPage);
        Event nextEvent = new Event(eventNameActionComplete, eventData);
        EventResult result = new EventResult(
                "Uploaded file " + file.getName() + " to " + docPage,
                nextEvent);
        return result;
    }
}
