/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.site;

import java.io.File;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.site.UpdateFilePage;
import org.alfresco.po.share.site.document.DocumentDetailsPage;
import org.alfresco.po.share.site.document.DocumentLibraryPage;

/**
 * Uploads a new version of a file from the document details page.
 *  
 * <h1>Input</h1> 
 * 
 * {@link DocumentDetailsPage} The page object showing the document details.
 * 
 * <h1>Actions</h1>
 * 
 * Populates the file upload form with file name and submits it to upload a new version of a document.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_FILE_UPLOAD_NEW_VERSION_COMPLETE}: {@link DocumentLibraryPage} in {@link ShareEventData}<br/>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class FileUploadNewVersionEventProcess extends AbstractShareEventProcessor
{
    public static final String EVENT_NAME_FILE_UPLOAD_NEW_VERSION_COMPLETE = "share.docDetails.uploadNewVersion.complete";
    
    /**
     * @param userDataService           service to provide user data
     */
    public FileUploadNewVersionEventProcess(UserDataService userDataService)
    {
        super(userDataService,EVENT_NAME_FILE_UPLOAD_NEW_VERSION_COMPLETE);
    }
    
    /**
     * Process which fills and submits the file upload form.
     * 
     * @param eventData {@link Event} event information
     * @return {@link EventResult} result of the process
     */
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        String filename = eventData.getFilename();
        if (filename == null)
        {
            throw new RuntimeException("No file provided to upload new version: " + eventData);
        }
        File file = new File(filename);
        // Make sure the file exists
        if (!file.exists())
        {
            throw new IllegalArgumentException("Upload file does not exist: " + file);
        }
        String filePath = file.getCanonicalPath();
        
        UpdateFilePage updateFilePage = (UpdateFilePage) eventData.getSharePage();
        
        updateFilePage.setComment("Update by session " + event.getSessionId());
        if (Math.random() > 0.1)
        {
            updateFilePage.selectMinorVersionChange();
        }
        else
        {
            updateFilePage.selectMajorVersionChange();
        }
        updateFilePage.uploadFile(filePath);
        DocumentDetailsPage docDetailsPage = updateFilePage.submitUpload().render();
        
        eventData.setSharePage(docDetailsPage);
        Event nextEvent = new Event(eventNameActionComplete, eventData);
        EventResult result = new EventResult(
                "Uploaded file " + file.getName() + " to " + updateFilePage,
                nextEvent);
        return result;
    }
}
