/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.site;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.process.share.entity.ShareSelectSiteEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.DashBoardPage;
import org.alfresco.po.share.ShareLink;
import org.alfresco.po.share.site.SiteDashboardPage;

/**
 * Navigate to Alfresco share site, mimics the action of entering a site by selecting the site
 * from the dashlet on the dashboard page.
 *  
 * <h1>Input</h1> 
 * 
 * {@link ShareEventData} containing:<br/>
 * {@link DashBoardPage} and {@link ShareEventData#getSiteName() a site name}.
 * 
 * <h1>Actions</h1>
 * 
 * Searches the link to the site from the dashlet present on the dashboard page, once
 * link is found it is selected to render the site dashboard page.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_NAVIGATE_TO_SITE_COMPLETE}: {@link SiteDashboardPage} in {@link ShareEventData}<br/>
 * 
 * @author Michael Suzuki
 * 
 * @since 1.0
 */
public class NavigateToSiteEventProcess extends AbstractShareEventProcessor
{
    private static final String EVENT_NAME_NAVIGATE_TO_SITE_COMPLETE = "share.navigateTo.site.complete";

    public NavigateToSiteEventProcess(UserDataService userDataService)
    {
        super(userDataService,EVENT_NAME_NAVIGATE_TO_SITE_COMPLETE);
    }
    
    /**
     * Process which looks for the site link on the dashlet and clicks it
     * to redirect user to the site page.
     * 
     * @param eventData {@link Event} event information
     * @return {@link EventResult} result of the process
     */
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        ShareSelectSiteEventData selectSiteEventData = (ShareSelectSiteEventData) eventData;
        ShareLink siteLink = selectSiteEventData.getSiteLink();
        
        // Click it
        SiteDashboardPage siteDashboardPage = siteLink.click().render();
        eventData.setSharePage(siteDashboardPage);
        Event nextEvent = new Event(eventNameActionComplete, selectSiteEventData);
        EventResult result = new EventResult(
                "Navigated to site: " + siteLink,
                nextEvent);
        return result;
    }
}
