/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.site;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.site.SiteFinderPage;

/**
 * An Alfresco share site delete process that deletes a site using the link on the {@link SiteFinderPage site finder}.
 *  
 * <h1>Input</h1>
 * 
 * {@link ShareEventData} containing the {@link ShareEventData#getSiteName() site name} and
 * {@link SiteFinderPage page} required to do the action.
 * 
 * <h1>Actions</h1>
 * 
 * Search for site by navigating to site finder and enters the site name to search.
 * If result of the search contains the site name the web driver then selects the
 * delete button set next to the result.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_SITE_DELETED}: {@link ShareEventData} containing {@link SiteFinderPage}.
 * 
 * @author Michael Suzuki
 * @since 1.0
 */
public class SiteDeleteEventProcess extends AbstractShareEventProcessor
{
    private static final String ERROR_MISSING_SITENAME = "Delete site action failed, siteName is required";
    private static final String ERROR_SITE_NAME_NOT_FOUND = "Delete site action failed, no sites matched the siteName";
    private static final String EVENT_NAME_SITE_DELETED = "share.site.deleted";
    
    public SiteDeleteEventProcess(UserDataService userDataService)
    {
        super(userDataService,EVENT_NAME_SITE_DELETED);
    }
    
    /**
     * Process which actions the delete site button.
     * 
     * @param eventData {@link Event} event information
     * @return {@link EventResult} result of the process
     */
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        String siteName = eventData.getSiteName();
        if (siteName == null)
        {
            return new EventResult(ERROR_MISSING_SITENAME, false);
        }
        SiteFinderPage siteFinder = (SiteFinderPage) eventData.getSharePage();
        if(siteFinder.hasResults())
        {
            siteFinder = siteFinder.deleteSite(siteName).render();
            eventData.setSharePage(siteFinder);
            Event nextEvent = new Event(eventNameActionComplete, eventData);
            EventResult result = new EventResult(
                    "Deleted site: " + siteName,
                    nextEvent);
            return result;
        }
        else
        {
            return new EventResult(ERROR_SITE_NAME_NOT_FOUND, false);
        }
    }
}
