/*
 * Copyright (C) 2005-2012 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.site;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.process.share.AbstractShareEventProcessor;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.site.SiteFinderPage;
import org.alfresco.po.HtmlPage;

/**
 * An Alfresco share site finder process, mimics the action of populating 
 * the site finder form with the site name to find the particular site.
 *  
 * <h1>Input</h1> 
 * 
 * {@link ShareEventData#getSiteName()}
 * 
 * <h1>Actions</h1>
 * 
 * Populates the site finder form with siteName and submits it to display results.
 * 
 * <h1>Output</h1>
 * 
 * {@link #EVENT_NAME_SITE_FOUND}: {@link SiteFinderPage} in {@link ShareEventData}<br/>
 * 
 * @author Michael Suzuki
 * 
 * @since 1.0
 */
public class SiteFinderEventProcess extends AbstractShareEventProcessor
{
    private static final String ERROR_MISSING_SITENAME = "find site action failed, siteName is required";
    private static final String EVENT_NAME_SITE_FOUND = "share.site.found";

    public SiteFinderEventProcess(UserDataService userDataService)
    {
        super(userDataService,EVENT_NAME_SITE_FOUND);
    }
    
    /**
     * Process which fills and submits the site finder form.
     * 
     * @param eventData {@link Event} event information
     * @return {@link EventResult} result of the process
     */
    @Override
    public EventResult processEvent(Event event, ShareEventData eventData) throws Exception
    {
        String siteName = eventData.getSiteName();
        if (siteName == null)
        {
            return new EventResult(ERROR_MISSING_SITENAME, false);
        }
        SiteFinderPage siteFinder = (SiteFinderPage) eventData.getSharePage();
        HtmlPage response = siteFinder.searchForSite(siteName);
        siteFinder = response.render();
        eventData.setSharePage(siteFinder);
        
        EventResult result = null;
        Event nextEvent = new Event(eventNameActionComplete, eventData);
        if (siteFinder.hasResults())
        {
            result = new EventResult(
                    "Got results for site search: " + siteName,
                    nextEvent);
        }
        else
        {
            // No results for search
            if (eventData instanceof ShareEventData.MustFindSite)
            {
                result = new EventResult(
                        "Site not found: " + siteName,
                        false);
            }
            else
            {
                // No results, but search was not expecting any
                result = new EventResult(
                        "Site not found (OK): " + siteName,
                        nextEvent);
            }
        }
        return result;
    }
}
