/*
 * Copyright (C) 2005-2013 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.soak;

import java.util.Collections;
import java.util.List;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.process.share.entity.ShareSelectSiteEventData;
import org.alfresco.po.share.DashBoardPage;
import org.alfresco.po.share.ShareLink;
import org.alfresco.po.share.SharePage;
import org.alfresco.po.share.dashlet.Dashlet;
import org.alfresco.po.share.dashlet.MySitesDashlet;
import org.alfresco.po.share.site.SiteDashboardPage;

/**
 * Browses to the site dashboard.
 * <p/>
 * The following page sequence is executed.  <b>*</b> indicates a possible start location.
 * <ul>
 *   <li><b>*  1. Any page after login (except a site dashboard):</b></li>
 *   <li><b>*  2. User dashboard:</b> A site is selected from the 'My Sites' dashlet</li>
 *   <li><b>   3. Site dashboard:</b></li>
 * </ul>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class ShareBrowseToSiteDashboardScenario extends ShareBrowseToUserDashboardScenario
{
    public static final String EVENT_NAME_NAVIGATE_TO_SITE = "share.navigateTo.site";

    private String eventNameNavigateToSite = EVENT_NAME_NAVIGATE_TO_SITE;
    
    /**
     * @param registry              registry with which this instance will be registered
     * @param weight                the weight relative to other scenarios registered in the registry
     */
    public ShareBrowseToSiteDashboardScenario(ShareScenarioRegistry registry, double weight)
    {
        super(registry, weight);
    }

    /**
     * Override the {@link #EVENT_NAME_NAVIGATE_TO_SITE default} name for events
     * to navigate to a specific site
     */
    public void setEventNameNavigateToSite(String eventNameNavigateToSite)
    {
        this.eventNameNavigateToSite = eventNameNavigateToSite;
    }

    /**
     * This instance first looks for the presence of the dashboard (step <b>2</b>)
     * and contues from there, if possible.  If not, the base class is called to first
     * execute a browse to the dashboard (step <b>1</b>).
     */
    @Override
    public List<Event> getNextEvents(
            Event event,
            ShareEventData eventData,
            ShareSoakData soakData,
            long nextEventTime)
    {
        SharePage sharePage = eventData.getSharePage();
        if (sharePage instanceof SiteDashboardPage)
        {
            // We are in a site already
            return Collections.emptyList();
        }
        else if (eventData instanceof ShareSelectSiteEventData)
        {
            // We've already done a selection
            return Collections.emptyList();
        }
        else if (sharePage instanceof DashBoardPage)
        {
            // We are in the dashboard.  Pick up a site name.
            DashBoardPage dashboardPage = (DashBoardPage) sharePage;
            Dashlet dashlet = dashboardPage.getDashlet("my-sites");
            if (dashlet == null)
            {
                // The user has no such dashlet
                return Collections.emptyList();
            }
            MySitesDashlet mySitesDashlet = dashlet.render();       // ClassCastException is OK
            List<ShareLink> siteLinks = mySitesDashlet.getSites();
            if (siteLinks.size() == 0)
            {
                // There are no sites.
                return Collections.emptyList();
            }
            int select = (int) (Math.random() * (double) siteLinks.size());
            ShareLink siteLink = siteLinks.get(select);
            // That's the one we'll go for
            ShareSelectSiteEventData selectSiteEventData = new ShareSelectSiteEventData(eventData, siteLink);
            // Create future event
            Event nextEvent = new Event(eventNameNavigateToSite, nextEventTime, selectSiteEventData);
            // Done
            return Collections.singletonList(nextEvent);
        }
        else
        {
            // Defer to the base implementation to take care of getting to the dashboard
            return super.getNextEvents(event, eventData, soakData, nextEventTime);
        }
    }
}
