/*
 * Copyright (C) 2005-2013 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.soak;

import java.util.Collections;
import java.util.List;
import java.util.UUID;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.process.share.entity.ShareCreateFolderEventData;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.po.share.SharePage;
import org.alfresco.po.share.site.NewFolderPage;
import org.alfresco.po.share.site.document.DocumentLibraryNavigation;
import org.alfresco.po.share.site.document.DocumentLibraryPage;

/**
 * Browses to a site document library.
 * <p/>
 * This class uses the {@link ShareBrowseToSiteDashboardScenario parent} scenario
 * until the site document library has been reached.
 * <p/>
 * The following page is executed.  <b>*</b> indicates a possible start location.
 * <ul>
 *   <li><b>*  1. Site doclib:</b> Selects the Upload link</li>
 *   <li><b>   2. File upload page:</b> Uploads a file with random details</li>
 *   <li><b>   3. Site doclib:</b> END</li>
 * </ul>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class ShareCreateFolderScenario extends ShareBrowseToSiteDocLibScenario
{
    public static final String EVENT_NAME_SELECT_NEW_FOLDER = "share.doclib.selectNewFolder";
    public static final String EVENT_NAME_CREATE_FOLDER = "share.doclib.createFolder";

    private String eventNameSelectNewFolder = EVENT_NAME_SELECT_NEW_FOLDER;
    private String eventNameCreateFolder = EVENT_NAME_CREATE_FOLDER;
    
    /**
     * @param registry              registry with which this instance will be registered
     * @param weight                the weight relative to other scenarios registered in the registry
     */
    public ShareCreateFolderScenario(ShareScenarioRegistry registry, double weight)
    {
        super(registry, weight);
    }

    /**
     * Override the {@link #EVENT_NAME_SELECT_NEW_FOLDER default} event name to open
     * the <i>New Folder</i> link.
     */
    public void setEventNameSelectNewFolder(String eventNameSelectNewFolder)
    {
        this.eventNameSelectNewFolder = eventNameSelectNewFolder;
    }

    /**
     * Override the {@link #EVENT_NAME_CREATE_FOLDER default} event name to create the folder
     */
    public void setEventNameCreateFolder(String eventNameCreateFolder)
    {
        this.eventNameCreateFolder = eventNameCreateFolder;
    }

    /**
     * @see ShareCreateSiteScenario see class comments
     */
    @Override
    public List<Event> getNextEvents(
            Event event,
            ShareEventData eventData,
            ShareSoakData soakData,
            long nextEventTime)
    {
        SharePage sharePage = eventData.getSharePage();
        if (sharePage instanceof DocumentLibraryPage)
        {
            // If we have just created a folder, we could be here.
            if (eventData instanceof ShareCreateFolderEventData)
            {
                return Collections.emptyList();
            }
            // Make sure that we can write to the location
            DocumentLibraryPage docLibPage = (DocumentLibraryPage) sharePage;
            DocumentLibraryNavigation docLibNav = docLibPage.getNavigation();
            if (!docLibNav.hasNewFolderLink())
            {
                // We can't do the action
                return Collections.emptyList();
            }
            // We need to click the New Folder link
            Event nextEvent = new Event(eventNameSelectNewFolder, nextEventTime, eventData);
            return Collections.singletonList(nextEvent);
        }
        else if (sharePage instanceof NewFolderPage)
        {
            // Choose a folder name
            String name = UUID.randomUUID().toString();
            eventData = new ShareCreateFolderEventData(eventData, name);
            // We need to do the creation
            Event nextEvent = new Event(eventNameCreateFolder, nextEventTime, eventData);
            return Collections.singletonList(nextEvent);
        }
        else
        {
            // Follow the parent class's actions
            return super.getNextEvents(event, eventData, soakData, nextEventTime);
        }
    }
}
