/*
 * Copyright (C) 2005-2013 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.soak;

import java.util.Collections;
import java.util.List;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.user.UserData;
import org.alfresco.bm.user.UserDataService;
import org.alfresco.po.share.PeopleFinderPage;
import org.alfresco.po.share.SharePage;

/**
 * Scenarios around people finder.
 * <p/>
 * The following page is executed.  <b>*</b> indicates a possible start location.
 * <ul>
 *   <li><b>*  1. Any page after login:</b> Select <i>People</i> link.</li>
 *   <li><b>*  2. {@link PeopleFinderPage} (no results):</b> Find random user and execute search according to
 *                                                           parameters of the scenario.</li>
 * </ul>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class SharePeopleFinderScenario extends AbstractShareScenario
{
    public static final String EVENT_NAME_NAVIGATE_TO_PEOPLE = "share.nav.people";
    public static final String EVENT_NAME_PEOPLE_SEARCH = "share.people.search";
    public static final boolean DEFAULT_PREFIX_SEARCH = true;

    private String eventNameNavigateToPeople = EVENT_NAME_NAVIGATE_TO_PEOPLE;
    private String eventNamePeopleSearch = EVENT_NAME_PEOPLE_SEARCH;
    private boolean prefixSearch = DEFAULT_PREFIX_SEARCH;
    private final UserDataService userDataService;
    
    /**
     * @param registry              registry with which this instance will be registered
     * @param weight                the weight relative to other scenarios registered in the registry
     * @param userDataService       service giving access to random usernames
     */
    public SharePeopleFinderScenario(ShareScenarioRegistry registry, double weight, UserDataService userDataService)
    {
        super(registry, weight);
        this.userDataService = userDataService;
    }

    /**
     * Override the {@link #EVENT_NAME_PEOPLE_SEARCH default} name for events
     * to navigate search on the <i>People Finder</i> screen.
     */
    public void setEventNamePeopleSearch(String eventNamePeopleSearch)
    {
        this.eventNamePeopleSearch = eventNamePeopleSearch;
    }

    /**
     * Override the {@link #EVENT_NAME_NAVIGATE_TO_PEOPLE default} name for events
     * to navigate to the <i>People Finder</i> screen.
     */
    public void setEventNameNavigateToPeople(String eventNameNavigateToPeople)
    {
        this.eventNameNavigateToPeople = eventNameNavigateToPeople;
    }

    /**
     * Override the {@link #DEFAULT_PREFIX_SEARCH default} flag indicating whether we perform
     * prefix searches or not.
     * 
     * @param prefixSearch          <tt>true</tt> to do a prefix search (<code>paul*</code> or
     *                              <tt>false</tt> to do a suffix search (<code>*paul</code).
     */
    public void setPrefixSearch(boolean prefixSearch)
    {
        this.prefixSearch = prefixSearch;
    }

    /**
     * If the browser is not on the {@link PeopleFinderPage} the link is clicked.<br/>
     */
    @Override
    public List<Event> getNextEvents(
            Event event,
            ShareEventData eventData,
            ShareSoakData soakData,
            long nextEventTime)
    {
        SharePage sharePage = eventData.getSharePage();
        if (sharePage instanceof PeopleFinderPage)
        {
            // Get a random user
            UserData user = userDataService.getRandomUser();
            String fullSearchString = null;
            double typeChance = Math.random();
            if (typeChance < 0.333)
            {
                fullSearchString = user.getUsername();
            }
            else if (typeChance < 0.666)
            {
                fullSearchString = user.getFirstName();
            }
            else
            {
                fullSearchString = user.getLastName();
            }
            // Make sure that we trim it
            if (fullSearchString != null)
            {
                fullSearchString = fullSearchString.trim().toLowerCase();
            }
            // Make sure that we have something
            if (fullSearchString == null || fullSearchString.length() == 0)
            {
                throw new RuntimeException("Found user without data to do a People Finder request: " + user);
            }
            int fullLength = fullSearchString.length();
            int searchLength = (int)(Math.random() * (double) fullLength);
            if (searchLength == 0)
            {
                searchLength = 3;
            }
            String searchString = null;
            if (prefixSearch)
            {
                searchString = fullSearchString.substring(0, searchLength);
            }
            else
            {
                searchString = "*" + fullSearchString.substring(fullLength - searchLength);
            }
            eventData.setSearch(searchString);
            Event nextEvent = new Event(eventNamePeopleSearch, nextEventTime, eventData);
            return Collections.singletonList(nextEvent);
        }
        else
        {
            // TODO: Remove hack after BENCH-141: WebDriver: PeopleFinderPage is not always produced with 'selectPeople'
            //      Use a token in the data to record that we have been here.
            if (eventData.getSearch() != null && eventData.getSearch().equals("@@BENCH-141"))
            {
                throw new IllegalStateException("BENCH-141: We are expecting a PeopleFinderPage but got " + sharePage.getClass());
            }
            eventData.setSearch("@@BENCH-141");
            
            Event nextEvent = new Event(eventNameNavigateToPeople, nextEventTime, eventData);
            return Collections.singletonList(nextEvent);
        }
    }
}
