/*
 * Copyright (C) 2005-2013 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.soak;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.file.TestFileService;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.process.share.entity.ShareSearchEventData;

/**
 * Executes an search using the general search box before widening the results and doing paging.
 * <p/>
 * The following page is executed.  <b>*</b> indicates a possible start location.
 * <ul>
 *   <li><b>*  1. Any page after login (including search results page):</b> Search string chosen (see notes)</li>
 *   <li><b>   2. Search results page:</b> </li>
 * </ul>
 * <b>Notes on search terms:</b><br/>
 * The {@link TestFileService} is used to find a file {@link #DEFAULT_SEARCH_TERMS_FILENAME <b>searchterms.txt</b>},
 * which is then read and random lines used (a search term per line).  If the file does not exist, some
 * {@link #DEFAULT_SEARCH_STRINGS default search strings} are chosen.
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class ShareSearchScenario extends AbstractShareScenario
{
    public static final String[] DEFAULT_SEARCH_STRINGS = new String[] {
            "\"quick\"",
            "Sa*",
            "alfresco",
            "ipsum",
            "\"ipsum lorum\""
            };
    public static final String DEFAULT_SEARCH_TERMS_FILENAME = "searchterms.txt";
    public static final String EVENT_NAME_SEARCH = "share.search";

    public String[] searchStrings;
    private TestFileService testFileService;
    private String searchTermsFilename = DEFAULT_SEARCH_TERMS_FILENAME;
    private String eventNameSearch = EVENT_NAME_SEARCH;
    
    /**
     * @param registry              registry with which this instance will be registered
     * @param weight                the weight relative to other scenarios registered in the registry
     */
    public ShareSearchScenario(ShareScenarioRegistry registry, double weight)
    {
        super(registry, weight);
    }

    /**
     * Optionally set the test file service to locate the {@link #setSearchTermsFilename(String)}
     * file containing search terms to use.  If this property is set, the file must be accessible.
     * 
     * @param testFileService               service providing remote access to test files
     */
    public void setTestFileService(TestFileService testFileService)
    {
        this.testFileService = testFileService;
    }

    /**
     * Override the {@link #DEFAULT_SEARCH_TERMS_FILENAME default} test filename containing search
     * terms (one on each line).
     * @param searchTermsFilename           the name of the remote file in the test file list
     */
    public void setSearchTermsFilename(String searchTermsFilename)
    {
        this.searchTermsFilename = searchTermsFilename;
    }

    /**
     * Override the {@link #EVENT_NAME_SEARCH default} name for the search event
     */
    public void setEventNameSearch(String eventNameSearch)
    {
        this.eventNameSearch = eventNameSearch;
    }

    /**
     * If the browser is <b>not</b> on the user dashboard then an event is fired to navigate to
     * the user dashboard.  If the user dashboard is the currrent page then nothing is done.
     */
    @Override
    public List<Event> getNextEvents(
            Event event,
            ShareEventData eventData,
            ShareSoakData soakData,
            long nextEventTime)
    {
        if (eventData instanceof ShareSearchEventData)
        {
            // We've just done a search, so choose a new scenario
            return Collections.emptyList();
        }
        
        // Choose a search string
        String[] searchStrings = getSearchStrings();
        String search = getRandomSearchString(searchStrings);
        ShareSearchEventData searchEventData = new ShareSearchEventData(eventData, search);
        // Raise next event
        Event nextEvent = new Event(eventNameSearch, nextEventTime, searchEventData);
        return Collections.singletonList(nextEvent);
    }
    
    /**
     * Safe method to get the search strings.  Blocking will be short-lived.
     */
    private synchronized String[] getSearchStrings()
    {
        if (searchStrings != null)
        {
            return searchStrings;
        }
        // No files.  Go straight to the defaults.
        if (testFileService == null)
        {
            searchStrings = DEFAULT_SEARCH_STRINGS;
        }
        else
        {
            searchStrings = getSearchStrings(testFileService, searchTermsFilename);
            if (searchStrings.length == 0)
            {
                searchStrings = DEFAULT_SEARCH_STRINGS;
            }
        }
        return searchStrings;
    }
    
    /**
     * Static helper to find and extract a search term from the given file
     * 
     * @param testFileService           the test files
     * @param searchTermsFilename       the name of the file to find
     * @return                          a list of search terms (empty if there are none or the remote file does not exist)
     */
    public static String[] getSearchStrings(TestFileService testFileService, String searchTermsFilename)
    {
        File file = testFileService.getFileByName(searchTermsFilename);
        if (file == null)
        {
            return new String[0];
        }
        FileReader fr = null;
        try
        {
            fr = new FileReader(file);
            BufferedReader br = new BufferedReader(fr);
            ArrayList<String> strings = new ArrayList<String>(500);
            String line;
            while ((line = br.readLine()) != null)
            {
                strings.add(line);
            }
            // Check
            if (strings.size() == 0)
            {
                throw new RuntimeException("No search strings in file: " + searchTermsFilename);
            }
            // Push into an array
            return (String[]) strings.toArray(new String[strings.size()]);
        }
        catch (IOException e)
        {
            throw new RuntimeException(
                    "Failed to read search strings from file '" + file +
                    "' loaded as '" + searchTermsFilename);
        }
        finally
        {
            if (fr != null)
            {
                try { fr.close(); } catch (IOException e) {}
            }
        }
    }
    
    private static final Random RANDOM = new Random();
    /**
     * Choose a random string from the search strings provided
     */
    public static String getRandomSearchString(String[] searchStrings)
    {
        if (searchStrings == null || searchStrings.length == 0)
        {
            throw new IllegalArgumentException("No search strings to choose from.");
        }
        return searchStrings[RANDOM.nextInt(searchStrings.length)];
    }
}
