/*
 * Copyright (C) 2005-2013 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.soak;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.List;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.file.TestFileService;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.bm.process.share.entity.ShareUploadFileEventData;
import org.alfresco.po.share.SharePage;
import org.alfresco.po.share.site.UploadFilePage;
import org.alfresco.po.share.site.document.DocumentLibraryNavigation;
import org.alfresco.po.share.site.document.DocumentLibraryPage;

/**
 * Uploads a file to the document library.
 * <p/>
 * This class uses the {@link ShareBrowseToSiteDashboardScenario parent} scenario
 * until the site document library has been reached.
 * <p/>
 * The following page is executed.  <b>*</b> indicates a possible start location.
 * <ul>
 *   <li><b>*  1. Site doclib:</b> Selects the Upload link</li>
 *   <li><b>   2. File upload page:</b> Uploads the selected file</li>
 *   <li><b>   3. Site doclib:</b> END</li>
 * </ul>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class ShareUploadFileScenario extends ShareBrowseToSiteDocLibScenario
{
    public static final String EVENT_NAME_SELECT_FILE_UPLOAD = "share.doclib.selectFileUpload";
    public static final String EVENT_NAME_UPLOAD_FILE = "share.doclib.uploadFile";

    private TestFileService testFileService;
    private String eventNameSelectFileUpload = EVENT_NAME_SELECT_FILE_UPLOAD;
    private String eventNameUploadFile = EVENT_NAME_UPLOAD_FILE;
    
    /**
     * @param registry              registry with which this instance will be registered
     * @param weight                the weight relative to other scenarios registered in the registry
     * @param testFileService       the provider of test files
     */
    public ShareUploadFileScenario(ShareScenarioRegistry registry, double weight, TestFileService testFileService)
    {
        super(registry, weight);
        this.testFileService = testFileService;
    }

    /**
     * Override the {@link #EVENT_NAME_SELECT_FILE_UPLOAD default} event name to open
     * the file upload page
     */
    public void setEventNameSelectFileUpload(String eventNameSelectFileUpload)
    {
        this.eventNameSelectFileUpload = eventNameSelectFileUpload;
    }

    /**
     * Override the {@link #EVENT_NAME_UPLOAD_FILE default} event name to upload a file
     */
    public void setEventNameUploadFile(String eventNameUploadFile)
    {
        this.eventNameUploadFile = eventNameUploadFile;
    }

    /**
     * @see ShareCreateSiteScenario see class comments
     */
    @Override
    public List<Event> getNextEvents(
            Event event,
            ShareEventData eventData,
            ShareSoakData soakData,
            long nextEventTime)
    {
        SharePage sharePage = eventData.getSharePage();
        if (sharePage instanceof DocumentLibraryPage)
        {
            // If we have just come out of the upload process, we could be here.
            // Check if the filename is still present.  If so, we're done.
            if (eventData instanceof ShareUploadFileEventData)
            {
                return Collections.emptyList();
            }
            // Make sure that we can write to the location
            DocumentLibraryPage docLibPage = (DocumentLibraryPage) sharePage;
            DocumentLibraryNavigation docLibNav = docLibPage.getNavigation();
            if (!docLibNav.hasFileUploadLink())
            {
                // We can't do the action
                return Collections.emptyList();
            }
            // We need to click the Upload link
            Event nextEvent = new Event(eventNameSelectFileUpload, nextEventTime, eventData);
            return Collections.singletonList(nextEvent);
        }
        else if (sharePage instanceof UploadFilePage)
        {
            // Choose a file
            File file = testFileService.getFile();
            if (file == null)
            {
                throw new RuntimeException("There are no test files available: " + testFileService);
            }
            try
            {
                eventData = new ShareUploadFileEventData(eventData, file.getCanonicalPath());
            }
            catch (IOException e)
            {
                throw new RuntimeException("Failed to get path for file: " + file);
            }
            // We need to actually upload a file
            Event nextEvent = new Event(eventNameUploadFile, nextEventTime, eventData);
            return Collections.singletonList(nextEvent);
        }
        else
        {
            // Follow the parent class's actions
            return super.getNextEvents(event, eventData, soakData, nextEventTime);
        }
    }
}
