/*
 * Copyright (C) 2005-2013 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.bm.process.share.soak;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.List;

import org.alfresco.bm.event.Event;
import org.alfresco.bm.file.FileData;
import org.alfresco.bm.file.TestFileService;
import org.alfresco.bm.process.share.entity.ShareEventData;
import org.alfresco.po.share.SharePage;
import org.alfresco.po.share.site.UpdateFilePage;
import org.alfresco.po.share.site.document.DocumentDetailsPage;

/**
 * Uploads a new version of a file.
 * <p/>
 * The following page is executed.  <b>*</b> indicates a possible start location.
 * <ul>
 *   <li><b>*  1. {@link DocumentDetailsPage}:</b> Selects the <i>Upload New Version</i> link</li>
 *   <li><b>*  2. {@link UpdateFilePage}:</b> Uploads a new version of the file</li>
 *   <li><b>   3. {@link DocumentDetailsPage}:</b> END</li>
 * </ul>
 * 
 * @author Derek Hulley
 * @since 1.0
 */
public class ShareUploadNewVersionScenario extends AbstractShareScenario
{
    public static final String EVENT_NAME_SELECT_UPLOAD_NEW_VERSION = "share.docDetails.selectUploadNewVersion";
    public static final String EVENT_NAME_UPLOAD_NEW_VERSION = "share.docDetails.uploadNewVersion";

    private TestFileService testFileService;
    private String eventNameSelectUploadNewVersion = EVENT_NAME_SELECT_UPLOAD_NEW_VERSION;
    private String eventNameUploadNewVersion = EVENT_NAME_UPLOAD_NEW_VERSION;
    
    /**
     * @param registry              registry with which this instance will be registered
     * @param weight                the weight relative to other scenarios registered in the registry
     * @param testFileService       the provider of test files
     */
    public ShareUploadNewVersionScenario(ShareScenarioRegistry registry, double weight, TestFileService testFileService)
    {
        super(registry, weight);
        this.testFileService = testFileService;
    }

    /**
     * Override the {@link #EVENT_NAME_SELECT_UPLOAD_NEW_VERSION default} event name to open
     * the new version upload page.
     */
    public void setEventNameSelectUploadNewVersion(String eventNameSelectUploadNewVersion)
    {
        this.eventNameSelectUploadNewVersion = eventNameSelectUploadNewVersion;
    }

    /**
     * Override the {@link #EVENT_NAME_UPLOAD_NEW_VERSION default} event name to upload
     * a new version of the file
     */
    public void setEventNameUploadNewVersion(String eventNameUploadNewVersion)
    {
        this.eventNameUploadNewVersion = eventNameUploadNewVersion;
    }

    /**
     * @see ShareCreateSiteScenario see class comments
     */
    @Override
    public List<Event> getNextEvents(
            Event event,
            ShareEventData eventData,
            ShareSoakData soakData,
            long nextEventTime)
    {
        SharePage sharePage = eventData.getSharePage();
        if (sharePage instanceof DocumentDetailsPage)
        {
            DocumentDetailsPage docDetailsPage = (DocumentDetailsPage) sharePage;
            // If we have just come out of the upload new version process then the filename
            // will be set and we can just avoid it.
            if (eventData.getFilename() != null)
            {
                return Collections.emptyList();
            }
            // Make sure we have permission to do this
            if (!docDetailsPage.isUploadNewVersionDisplayed())
            {
                // File is read-only
                return Collections.emptyList();
            }
            // Get the name of the file so that we can select the correct type for later updates
            String filename = docDetailsPage.getDocumentTitle();
            if(filename == null || filename.isEmpty())
            {
                return Collections.emptyList();
            }
            eventData.setFilename(filename);
            // We need to click the Upload New Version link
            Event nextEvent = new Event(eventNameSelectUploadNewVersion, nextEventTime, eventData);
            return Collections.singletonList(nextEvent);
        }
        else if (sharePage instanceof UpdateFilePage)
        {
            String filename = eventData.getFilename();
            if (filename == null)
            {
                throw new IllegalStateException("Expected for the ShareData filename to contain the document title.");
            }
            String extension = FileData.getExtension(filename);
            // Choose a file
            File file = testFileService.getFile(extension);
            if (file == null)
            {
                throw new RuntimeException(
                        "There are no test files available for the given extension: \n" +
                        "   Extension:          " + extension + "\n" +
                        "   TestFileService:    " + testFileService);
            }
            try
            {
                eventData.setFilename(file.getCanonicalPath());
            }
            catch (IOException e)
            {
                throw new RuntimeException("Failed to get path for file: " + file);
            }
            // We need to actually upload a file
            Event nextEvent = new Event(eventNameUploadNewVersion, nextEventTime, eventData);
            return Collections.singletonList(nextEvent);
        }
        else
        {
            // This only works on the document details page
            return Collections.emptyList();
        }
    }
}
